@@ -3,16 +3,12 @@
 
 int main(void)
 {
-    cc_param_t param;
     pairing_t pairing;
     element_t x, y, r;
     int i, n;
     double t0, t1, ttotal;
 
-    cc_param_init(param);
-    cc_param_inp_str(param, stdin);
-
-    pairing_init_cc_param(pairing, param);
+    pairing_init_inp_str(pairing, stdin);
     element_init(x, pairing->G1);
     element_init(y, pairing->G2);
     element_init(r, pairing->GT);
