@@ -563,20 +563,20 @@ static void cc_tatepower_even_k(element_ptr out, element_ptr in, pairing_t pairi
 	element_init(e3, p->Fqk);
 	element_ptr e0re = fi_re(e0);
 	element_ptr e0im = fi_im(e0);
-	element_ptr inre = fi_re(in);
-	element_ptr inim = fi_im(in);
+	element_t *inre = fi_re(in)->data;
+	element_t *inim = fi_im(in)->data;
 	void subexpr(element_ptr e) {
 	    element_set0(e0);
-	    poly_set_coeff(e0re, poly_coeff(inre, 0), 0);
-	    poly_set_coeff(e0im, poly_coeff(inim, 0), 0);
-	    poly_const_mul(e2, poly_coeff(inre, 1), e);
+	    element_set(((element_t *) e0re->data)[0], inre[0]);
+	    element_set(((element_t *) e0im->data)[0], inim[0]);
+	    polymod_const_mul(e2, inre[1], e);
 	    element_add(e0re, e0re, e2);
-	    poly_const_mul(e2, poly_coeff(inim, 1), e);
+	    polymod_const_mul(e2, inim[1], e);
 	    element_add(e0im, e0im, e2);
 	    element_mul(e1, e, e);
-	    poly_const_mul(e2, poly_coeff(inre, 2), e1);
+	    polymod_const_mul(e2, inre[2], e1);
 	    element_add(e0re, e0re, e2);
-	    poly_const_mul(e2, poly_coeff(inim, 2), e1);
+	    polymod_const_mul(e2, inim[2], e1);
 	    element_add(e0im, e0im, e2);
 	}
 	subexpr(p->xpowq[4]);
@@ -615,6 +615,8 @@ static void cc_pairing_even_k(element_ptr out, element_ptr in1, element_ptr in2,
     element_mul(fi_im(y), Qbase->y, p->nqrinv2);
     cc_miller_no_denom(out, pairing->r, in1->data, x, y, p->mapbase);
     cc_tatepower_even_k(out, out, pairing);
+    element_clear(x);
+    element_clear(y);
 }
 
 static int cc_is_almost_coddh_even_k(element_ptr a, element_ptr b,
@@ -669,7 +671,7 @@ static int cc_is_almost_coddh_even_k(element_ptr a, element_ptr b,
 
 static void Fq_to_Fqd_to_Fqk(element_t out, element_t in)
 {
-    element_field_to_poly(fi_re(out), in);
+    element_field_to_polymod(fi_re(out), in);
     element_set0(fi_im(out));
 }
 
@@ -709,8 +711,7 @@ static void pairing_init_c_param_even_k(pairing_t pairing, c_param_t param)
 
     p->Fqd->nqr = malloc(sizeof(element_t));
     element_init(p->Fqd->nqr, p->Fqd);
-    poly_alloc(p->Fqd->nqr, 1);
-    element_set_mpz(poly_coeff(p->Fqd->nqr, 0), param->nqr);
+    element_set_mpz(((element_t *) p->Fqd->nqr->data)[0], param->nqr);
 
     field_init_quadratic(p->Fqk, p->Fqd);
 
@@ -726,7 +727,7 @@ static void pairing_init_c_param_even_k(pairing_t pairing, c_param_t param)
 
 	p->xpowq = malloc(sizeof(element_ptr) * 5);
 	element_init(e0, p->Fqd);
-	poly_setx(e0);
+	element_set1(((element_t *) e0->data)[1]);
 	for (i=1; i<=4; i++) {
 	    element_ptr e = p->xpowq[i] = malloc(sizeof(element_t));
 	    element_init(e, p->Fqd);
@@ -742,7 +743,7 @@ static void pairing_init_c_param_even_k(pairing_t pairing, c_param_t param)
 
     p->mapbase = Fq_to_Fqd_to_Fqk;
 
-    cc_init_map_curve(p->Etwist, p->Eq, p->Fqd, element_field_to_poly);
+    cc_init_map_curve(p->Etwist, p->Eq, p->Fqd, element_field_to_polymod);
     twist_curve(p->Etwist);
     element_init(p->nqrinv, p->Fqd);
     element_invert(p->nqrinv, field_get_nqr(p->Fqd));
@@ -821,15 +822,13 @@ static void compute_cm_curve(c_param_ptr param, cm_info_ptr cm)
     darray_clear(coefflist);
     //TODO: remove x = 0, 1728 roots
     //TODO: what if there's no roots?
-    /*
-    printf("hp ");
-    element_out_str(stdout, hp);
-    printf("\n");
-    */
+    //printf("hp ");
+    //element_out_str(stdout, 0, hp);
+    //printf("\n");
     element_init(root, fp);
     findroot(root, hp);
     //printf("root = ");
-    //element_out_str(stdout, root);
+    //element_out_str(stdout, 0, root);
     //printf("\n");
     element_clear(hp);
     field_clear(fpx);
@@ -905,8 +904,7 @@ void c_param_from_cm(c_param_t param, cm_info_ptr cm)
     //that is not a square in Fqd
     element_init(nqr, Fqd);
     do {
-	poly_alloc(nqr, 1);
-	element_random(poly_coeff(nqr, 0));
+	element_random(((element_t *) nqr->data)[0]);
     } while (element_is_sqr(nqr));
 
     param->coeff = realloc(param->coeff, sizeof(mpz_t) * d);
@@ -915,7 +913,7 @@ void c_param_from_cm(c_param_t param, cm_info_ptr cm)
 	mpz_init(param->coeff[i]);
 	mpz_set(param->coeff[i], poly_coeff(irred, i)->data);
     }
-    mpz_set(param->nqr, poly_coeff(nqr, 0)->data);
+    mpz_set(param->nqr, ((element_t *) nqr->data)[0]->data);
 
     element_clear(nqr);
     element_clear(irred);
