@@ -445,3 +445,147 @@ void field_init_curve_group(field_t f, curve_t c, mpz_t cofac)
     f->to_bytes = curve_group_to_bytes;
     f->from_bytes = curve_group_from_bytes;
 }
+
+//singular with node: y^2 = x^3 + x^2
+static void sn_random(point_ptr p)
+{
+    element_t t;
+
+    element_init(t, p->curve->field);
+    p->inf_flag = 0;
+    do {
+	element_random(p->x);
+	if (element_is0(p->x)) continue;
+	element_mul(t, p->x, p->x);
+	element_add(t, t, p->x);
+	element_mul(t, t, p->x);
+    } while (!element_is_sqr(t));
+    element_sqrt(p->y, t);
+
+    element_clear(t);
+}
+
+static inline void sn_double_no_check(point_ptr r, point_ptr p)
+{
+    element_t lambda, e0, e1;
+
+    element_init(lambda, p->curve->field);
+    element_init(e0, p->curve->field);
+    element_init(e1, p->curve->field);
+    //same point: double them
+
+    //lambda = (3x^2 + 2x) / 2y
+    element_set_si(e0, 3);
+    element_mul(lambda, e0, p->x);
+    element_set_si(e0, 2);
+    element_add(lambda, lambda, e0);
+    element_mul(lambda, lambda, p->x);
+    element_add(e0, p->y, p->y);
+    element_invert(e0, e0);
+    element_mul(lambda, lambda, e0);
+    //x1 = lambda^2 - 2x - 1
+    element_add(e1, p->x, p->x);
+    element_mul(e0, lambda, lambda);
+    element_sub(e0, e0, e1);
+    element_set_si(e1, 1);
+    element_sub(e0, e0, e1);
+    //y1 = (x - x1)lambda - y
+    element_sub(e1, p->x, e0);
+    element_mul(e1, e1, lambda);
+    element_sub(e1, e1, p->y);
+
+    element_set(r->x, e0);
+    element_set(r->y, e1);
+    r->inf_flag = 0;
+
+    element_clear(lambda);
+    element_clear(e0);
+    element_clear(e1);
+    return;
+}
+
+static void sn_double(point_ptr r, point_ptr p)
+{
+    if (point_is_inf(p)) {
+	point_set_inf(r);
+	return;
+    }
+    if (element_is0(p->y)) {
+	point_set_inf(r);
+	return;
+    }
+    sn_double_no_check(r, p);
+}
+
+static void sn_add(point_ptr r, point_ptr p, point_ptr q)
+{
+    if (point_is_inf(p)) {
+	point_set(r, q);
+	return;
+    }
+    if (point_is_inf(q)) {
+	point_set(r, p);
+	return;
+    }
+    if (!element_cmp(p->x, q->x)) {
+	if (!element_cmp(p->y, q->y)) {
+	    if (element_is0(p->y)) {
+		point_set_inf(r);
+		return;
+	    } else {
+		sn_double_no_check(r, p);
+		return;
+	    }
+	}
+	//points are inverses of each other
+	point_set_inf(r);
+	return;
+    } else {
+	element_t lambda, e0, e1;
+
+	element_init(lambda, p->curve->field);
+	element_init(e0, p->curve->field);
+	element_init(e1, p->curve->field);
+
+	//lambda = (y2-y1)/(x2-x1)
+	element_sub(e0, q->x, p->x);
+	element_invert(e0, e0);
+	element_sub(lambda, q->y, p->y);
+	element_mul(lambda, lambda, e0);
+	//x3 = lambda^2 - x1 - x2 - 1
+	element_mul(e0, lambda, lambda);
+	element_sub(e0, e0, p->x);
+	element_sub(e0, e0, q->x);
+	element_set1(e1);
+	element_sub(e0, e0, e1);
+	//y3 = (x1-x3)lambda - y1
+	element_sub(e1, p->x, e0);
+	element_mul(e1, e1, lambda);
+	element_sub(e1, e1, p->y);
+
+	element_set(r->x, e0);
+	element_set(r->y, e1);
+        r->inf_flag = 0;
+
+	element_clear(lambda);
+	element_clear(e0);
+	element_clear(e1);
+    }
+}
+
+static void sn_clear(curve_ptr c)
+{
+}
+
+void curve_init_singular_with_node(curve_ptr c, field_t field)
+{
+    c->field = field;
+    c->random = sn_random;
+    //c->from_x = cc_from_x;
+    //c->from_hash = cc_from_hash;
+    c->neg = cc_neg;
+    c->doublefn = sn_double;
+    c->add = sn_add;
+    c->mul = cc_mul;
+    c->curve_clear = sn_clear;
+}
