@@ -128,26 +128,6 @@ struct common_curve_s {
 typedef struct common_curve_s common_curve_t[1];
 typedef struct common_curve_s *common_curve_ptr;
 
-struct cc_param_s {
-    mpz_t q; //curve defined over F_q
-    mpz_t n; //has order n (= q - t + 1) in F_q
-    mpz_t h; //h * r = n, r is prime
-    mpz_t r;
-    mpz_t a, b; //curve equation is y^2 = x^3 + ax + b
-    int k; //embedding degree
-    mpz_t nk; //order of curve over F_q^k
-    mpz_t hk; //hk * r^2 = nk
-};
-
-typedef struct cc_param_s cc_param_t[1];
-typedef struct cc_param_s *cc_param_ptr;
-
-void cc_param_init(cc_param_ptr cc);
-void cc_param_clear(cc_param_ptr cc);
-void cc_param_set(cc_param_ptr x, cc_param_ptr y);
-void cc_param_out_str(FILE *stream, cc_param_ptr p);
-void cc_param_inp_str(cc_param_ptr p, FILE *stream);
-
 //for shoehorning the group of points on a curve into the `field' data type
 struct curve_group_data_s {
     curve_ptr curve;
