@@ -9,6 +9,8 @@
 #include <stdlib.h>
 #include <gmp.h>
 
+#include "random.h"
+
 struct field_s;
 
 struct element_s {
@@ -194,8 +196,6 @@ static inline void field_clear(field_ptr f)
 
 element_ptr field_get_nqr(field_ptr f);
 
-void random_set_file(char *);
-
 static inline void field_init(field_ptr f)
 {
     f->nqr = NULL;
