@@ -105,6 +105,8 @@ static int zp_cmp(element_ptr a, element_ptr b)
 
 static int zp_is_sqr(element_ptr a)
 {
+    //0 is a square
+    if (!mpz_sgn((mpz_ptr) a->data)) return 1;
     return mpz_legendre(a->data, a->field->order) == 1;
 }
 
