@@ -212,24 +212,6 @@ static void poly_mul(element_ptr r, element_ptr f, element_ptr g)
     element_clear(prod);
 }
 
-void poly_const_mul(element_ptr res, element_ptr a, element_ptr poly)
-    //a lies in R, poly in R[x]
-{
-    int i;
-    darray_ptr coeff = ((poly_element_ptr) poly->data)->coeff;
-    int n = coeff->count;
-    element_t r;
-
-    element_init(r, res->field);
-    poly_alloc(r, n);
-    for (i=0; i<n; i++) {
-	element_mul(poly_coeff(r, i), coeff->item[i], a);
-    }
-    poly_remove_leading_zeroes(r);
-    element_set(res, r);
-    element_clear(r);
-}
-
 void poly_random_monic(element_ptr f, int deg)
 {
     int i;
@@ -240,54 +222,38 @@ void poly_random_monic(element_ptr f, int deg)
     element_set1(poly_coeff(f, i));
 }
 
-static void polymod_random(element_ptr f)
+static void polymod_random(element_ptr e)
 {
-    polymod_field_data_ptr p = f->field->data;
-    int n = poly_degree(p->poly);
-    int i;
-    poly_alloc(f, n);
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data;
+    int i, n = p->n;
+
     for (i=0; i<n; i++) {
-	element_random(poly_coeff(f, i));
+	element_random(coeff[i]);
     }
-    poly_remove_leading_zeroes(f);
 }
 
-static void polymod_from_hash(element_ptr f, int len, void *data)
+static void polymod_from_hash(element_ptr e, int len, void *data)
 {
-    polymod_field_data_ptr p = f->field->data;
-    int n = poly_degree(p->poly);
-    int i;
-    poly_alloc(f, n);
+    //TODO: improve this
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data;
+    int i, n = p->n;
     for (i=0; i<n; i++) {
-	element_from_hash(poly_coeff(f, i), len, data);
+	element_from_hash(coeff[i], len, data);
     }
-    poly_remove_leading_zeroes(f);
 }
 
-static void polymod_mul(element_ptr res, element_ptr f, element_ptr g)
+void polymod_const_mul(element_ptr res, element_ptr a, element_ptr e)
+    //a lies in R, e in R[x]
 {
-    element_t prod;
-    int i, n, n1;
-    polymod_field_data_ptr p = res->field->data;
-    element_t p0;
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data, *dst = res->data;
+    int i, n = p->n;
 
-    element_init(prod, res->field);
-    element_init(p0, res->field);
-    n = poly_degree(p->poly);
-
-    poly_mul(prod, f, g);
-
-    n1 = poly_degree(prod);
-    for (i=n; i<=n1; i++) {
-	poly_const_mul(p0, poly_coeff(prod, i), p->xpwr->item[i-n]);
-	element_add(prod, prod, p0);
+    for (i=0; i<n; i++) {
+	element_mul(dst[i], coeff[i], a);
     }
-    poly_alloc(prod, n);
-    poly_remove_leading_zeroes(prod);
-
-    element_set(res, prod);
-    element_clear(prod);
-    element_clear(p0);
 }
 
 static void poly_set0(element_ptr e)
@@ -389,6 +355,16 @@ void poly_div(element_ptr quot, element_ptr rem,
     element_clear(binv);
 }
 
+void poly_const_mul(element_ptr res, element_ptr a, element_ptr poly)
+{
+    int i, n = poly_coeff_count(poly);
+    poly_alloc(res, n);
+    for (i=0; i<n; i++) {
+	element_mul(poly_coeff(res, i), a, poly_coeff(poly, i));
+    }
+    poly_remove_leading_zeroes(res);
+}
+
 void poly_invert(element_ptr res, element_ptr f, element_ptr m)
 {
     element_t q, r0, r1, r2;
@@ -430,10 +406,48 @@ void poly_invert(element_ptr res, element_ptr f, element_ptr m)
     element_clear(b2);
 }
 
-static void polymod_invert(element_ptr r, element_ptr f)
+void element_polymod_to_poly(element_ptr f, element_ptr e)
+{
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data;
+    int i, n = p->n;
+    poly_alloc(f, n);
+    for (i=0; i<n; i++) {
+	element_set(poly_coeff(f, i), coeff[i]);
+    }
+    poly_remove_leading_zeroes(f);
+}
+
+void element_poly_to_polymod_truncate(element_ptr e, element_ptr f)
+{
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data;
+    int i;
+    int n;
+    n = poly_coeff_count(f);
+    if (n > p->n) n = p->n;
+
+    for (i=0; i<n; i++) {
+	element_set(coeff[i], poly_coeff(f, i));
+    }
+    for (; i<p->n; i++) {
+	element_set0(coeff[i]);
+    }
+}
+
+static void polymod_invert(element_ptr r, element_ptr e)
 {
     polymod_field_data_ptr p = r->field->data;
-    poly_invert(r, f, p->poly);
+    element_ptr minpoly = p->poly;
+    element_t f, r1;
+
+    element_init(f, minpoly->field);
+    element_init(r1, minpoly->field);
+    element_polymod_to_poly(f, e);
+
+    poly_invert(r1, f, p->poly);
+
+    element_poly_to_polymod_truncate(r, r1);
 }
 
 void element_field_to_poly(element_ptr f, element_ptr g)
@@ -443,6 +457,17 @@ void element_field_to_poly(element_ptr f, element_ptr g)
     poly_remove_leading_zeroes(f);
 }
 
+void element_field_to_polymod(element_ptr f, element_ptr g)
+{
+    polymod_field_data_ptr p = f->field->data;
+    element_t *coeff = f->data;
+    int i, n = p->n;
+    element_set(coeff[0], g);
+    for (i=1; i<n; i++) {
+	element_set0(coeff[i]);
+    }
+}
+
 static int poly_cmp(element_ptr f, element_ptr g)
 {
     int i;
@@ -491,13 +516,13 @@ void field_init_poly(field_ptr f, field_ptr base_field)
 static void field_clear_polymod(field_ptr f)
 {
     polymod_field_data_ptr p = f->data;
-    void clear(void *data) {
-	element_clear(data);
-	free(data);
+    int i, n = p->n;
+
+    for (i=0; i<n; i++) {
+	element_clear(p->xpwr[i]);
     }
+    free(p->xpwr);
 
-    darray_forall(p->xpwr, clear);
-    darray_clear(p->xpwr);
     element_clear(p->poly);
     free(f->data);
 }
@@ -520,7 +545,8 @@ static int polymod_is_sqr(element_ptr e)
 }
 
 static void polymod_sqrt(element_ptr res, element_ptr a)
-    //use Cantor-Zassenhaus
+    //use Cantor-Zassenhaus aka Legendre's method
+    //TODO: use another, faster method?
 {
     field_t kx;
     element_t f;
@@ -621,13 +647,11 @@ void poly_make_monic(element_t f, element_t g)
 static int polymod_to_bytes(unsigned char *data, element_t f)
 {
     polymod_field_data_ptr p = f->field->data;
-    int n = poly_degree(p->poly);
-    int i;
+    element_t *coeff = f->data;
+    int i, n = p->n;
     int len = 0;
-
     for (i=0; i<n; i++) {
-	element_ptr e = poly_coeff(f, i);
-	len += element_to_bytes(data + len, e);
+	len += element_to_bytes(data + len, coeff[i]);
     }
     return len;
 }
@@ -635,12 +659,12 @@ static int polymod_to_bytes(unsigned char *data, element_t f)
 static int polymod_length_in_bytes(element_t f)
 {
     polymod_field_data_ptr p = f->field->data;
-    int n = poly_degree(p->poly);
+    element_t *coeff = f->data;
+    int i, n = p->n;
     int res = 0;
-    int i;
 
     for (i=0; i<n; i++) {
-	res += element_length_in_bytes(poly_coeff(f, i));
+	res += element_length_in_bytes(coeff[i]);
     }
 
     return res;
@@ -649,48 +673,238 @@ static int polymod_length_in_bytes(element_t f)
 static int polymod_from_bytes(element_t f, unsigned char *data)
 {
     polymod_field_data_ptr p = f->field->data;
-    int n = poly_degree(p->poly);
+    element_t *coeff = f->data;
+    int i, n = p->n;
     int len = 0;
-    int i;
 
-    poly_alloc(f, n);
     for (i=0; i<n; i++) {
-	len += element_from_bytes(poly_coeff(f, i), data + len);
+	len += element_from_bytes(coeff[i], data + len);
     }
     return len;
 }
 
-void compute_x_powers(darray_ptr xpwr, element_ptr poly)
-//compute x^n,...,x^{2n-2} mod poly
+static void polymod_init(element_t e)
 {
-    element_t p0;
-    element_ptr pwr, pwr1, pwrn;
-    int i, j, k;
-    int n = poly_coeff_count(poly);
+    int i;
+    polymod_field_data_ptr p = e->field->data;
+    int n = p->n;
+    element_t *coeff;
+    coeff = e->data = malloc(sizeof(element_t) * n);
 
-    element_init(p0, poly->field);
-    for (i=0; i<n-1; i++) {
-	pwr = malloc(sizeof(element_t));
-	element_init(pwr, poly->field);
-	darray_append(xpwr, pwr);
+    for (i=0; i<n; i++) {
+	element_init(coeff[i], p->field);
     }
-    pwrn = xpwr->item[0];
-    element_neg(pwrn, poly);
-    poly_alloc(pwrn, n - 1);
-    poly_remove_leading_zeroes(pwrn);
-    for (i=1; i<n-1; i++) {
-	pwr = xpwr->item[i-1];
-	pwr1 = xpwr->item[i];
-	k = poly_coeff_count(pwr);
-	for (j=0; j<k; j++) {
-	    poly_set_coeff(pwr1, poly_coeff(pwr, j), j+1);
+}
+
+static void polymod_clear(element_t e)
+{
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data;
+    int i, n = p->n;
+    for (i=0; i<n; i++) {
+	element_clear(coeff[i]);
+    }
+    free(e->data);
+}
+
+static void polymod_set_si(element_t e, signed long int x)
+{
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data;
+    int i, n = p->n;
+    element_set_si(coeff[0], x);
+    for (i=1; i<n; i++) {
+	element_set0(coeff[i]);
+    }
+}
+
+static void polymod_set_mpz(element_t e, mpz_t z)
+{
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data;
+    int i, n = p->n;
+    element_set_mpz(coeff[0], z);
+    for (i=1; i<n; i++) {
+	element_set0(coeff[i]);
+    }
+}
+
+static void polymod_set(element_t e, element_t f)
+{
+    polymod_field_data_ptr p = e->field->data;
+    element_t *dst = e->data, *src = f->data;
+    int i, n = p->n;
+    for (i=0; i<n; i++) {
+	element_set(dst[i], src[i]);
+    }
+}
+
+static void polymod_neg(element_t e, element_t f)
+{
+    polymod_field_data_ptr p = e->field->data;
+    element_t *dst = e->data, *src = f->data;
+    int i, n = p->n;
+    for (i=0; i<n; i++) {
+	element_neg(dst[i], src[i]);
+    }
+}
+
+static int polymod_cmp(element_ptr f, element_ptr g)
+{
+    polymod_field_data_ptr p = f->field->data;
+    element_t *c1 = f->data, *c2 = g->data;
+    int i, n = p->n;
+    for (i=0; i<n; i++) {
+	if (element_cmp(c1[i], c2[i])) return 1;
+    }
+    return 0;
+}
+
+static void polymod_add(element_t r, element_t e, element_t f)
+{
+    polymod_field_data_ptr p = r->field->data;
+    element_t *dst = r->data, *s1 = e->data, *s2 = f->data;
+    int i, n = p->n;
+    for (i=0; i<n; i++) {
+	element_add(dst[i], s1[i], s2[i]);
+    }
+}
+
+static void polymod_sub(element_t r, element_t e, element_t f)
+{
+    polymod_field_data_ptr p = r->field->data;
+    element_t *dst = r->data, *s1 = e->data, *s2 = f->data;
+    int i, n = p->n;
+    for (i=0; i<n; i++) {
+	element_sub(dst[i], s1[i], s2[i]);
+    }
+}
+
+static void polymod_mul(element_ptr res, element_ptr e, element_ptr f)
+{
+    polymod_field_data_ptr p = res->field->data;
+    element_t *dst, *s1 = e->data, *s2 = f->data;
+    element_t prod, p0, c0;
+    int i, j, n = p->n;
+
+    element_init(prod, res->field);
+    dst = prod->data;
+    element_init(p0, res->field);
+    element_init(c0, p->field);
+
+    for (i=0; i<n; i++) {
+	//TODO: more efficient logic
+	for (j=0; j<n; j++) {
+	    element_mul(c0, s1[i], s2[j]);
+	    if (i + j < n) {
+		element_add(dst[i + j], dst[i + j], c0);
+	    } else {
+		polymod_const_mul(p0, c0, p->xpwr[i + j - n]);
+		element_add(prod, prod, p0);
+	    }
 	}
-	if (k == n - 1) {
-	    element_ptr e1 = poly_coeff(pwr1, k);
-	    poly_const_mul(p0, e1, pwrn);
-	    poly_alloc(pwr1, k);
-	    element_add(pwr1, pwr1, p0);
+    }
+
+    element_set(res, prod);
+    element_clear(prod);
+    element_clear(p0);
+    element_clear(c0);
+}
+
+static int polymod_is0(element_ptr e)
+{
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data;
+    int i, n = p->n;
+
+    for (i=0; i<n; i++) {
+	if (!element_is0(coeff[i])) return 0;
+    }
+    return 1;
+}
+
+static int polymod_is1(element_ptr e)
+{
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data;
+    int i, n = p->n;
+
+    if (!element_is1(coeff[0])) return 0;
+    for (i=1; i<n; i++) {
+	if (!element_is0(coeff[i])) return 0;
+    }
+    return 1;
+}
+
+static void polymod_set0(element_ptr e)
+{
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data;
+    int i, n = p->n;
+
+    for (i=0; i<n; i++) {
+	element_set0(coeff[i]);
+    }
+}
+
+static void polymod_set1(element_ptr e)
+{
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data;
+    int i, n = p->n;
+
+    element_set1(coeff[0]);
+    for (i=1; i<n; i++) {
+	element_set0(coeff[i]);
+    }
+}
+
+static void polymod_out_str(FILE *stream, int base, element_ptr e)
+{
+    polymod_field_data_ptr p = e->field->data;
+    element_t *coeff = e->data;
+    int i, n = p->n;
+
+    fputc('[', stream);
+    for (i=0; i<n; i++) {
+	if (i) fputc(' ', stream);
+	element_out_str(stream, base, coeff[i]);
+    }
+    fputc(']', stream);
+}
+
+static void compute_x_powers(field_ptr field, element_ptr poly)
+//compute x^n,...,x^{2n-2} mod poly
+//TODO: nest in field_init_polymod()?
+{
+    polymod_field_data_ptr p = field->data;
+    element_t p0;
+    element_ptr pwrn;
+    element_t *coeff, *coeff1;
+    int i, j;
+    int n = p->n;
+    element_t *xpwr;
+    
+    xpwr = p->xpwr;
+
+    element_init(p0, field);
+    for (i=0; i<n; i++) {
+	element_init(xpwr[i], field);
+    }
+    pwrn = xpwr[0];
+    element_poly_to_polymod_truncate(pwrn, poly);
+    element_neg(pwrn, pwrn);
+
+    for (i=1; i<n; i++) {
+	coeff = xpwr[i-1]->data;
+	coeff1 = xpwr[i]->data;
+
+	element_set0(coeff1[0]);
+	for (j=1; j<n; j++) {
+	    element_set(coeff1[j], coeff[j - 1]);
 	}
+	polymod_const_mul(p0, coeff[n - 1], pwrn);
+	element_add(xpwr[i], xpwr[i], p0);
     }
     element_clear(p0);
 }
@@ -699,9 +913,7 @@ void field_init_polymod(field_ptr f, element_ptr poly)
     //assumes poly is monic
 {
     poly_field_data_ptr pdp = poly->field->data;
-    poly_element_ptr pd = poly->data;
     polymod_field_data_ptr p;
-    int n;
 
     field_init(f);
     f->data = malloc(sizeof(polymod_field_data_t));
@@ -710,21 +922,22 @@ void field_init_polymod(field_ptr f, element_ptr poly)
     p->mapbase = element_field_to_poly;
     element_init(p->poly, poly->field);
     element_set(p->poly, poly);
+    p->n = poly_degree(p->poly);
     f->field_clear = field_clear_polymod;
-    f->init = poly_init;
-    f->clear = poly_clear;
-    f->set_si = poly_set_si;
-    f->set_mpz = poly_set_mpz;
-    f->out_str = poly_out_str;
-    f->set = poly_set;
-    f->add = poly_add;
-    f->is0 = poly_is0;
-    f->is1 = poly_is1;
-    f->set0 = poly_set0;
-    f->set1 = poly_set1;
-    f->sub = poly_sub;
-    f->neg = poly_neg;
-    f->cmp = poly_cmp;
+    f->init = polymod_init;
+    f->clear = polymod_clear;
+    f->set_si = polymod_set_si;
+    f->set_mpz = polymod_set_mpz;
+    f->out_str = polymod_out_str;
+    f->set = polymod_set;
+    f->add = polymod_add;
+    f->sub = polymod_sub;
+    f->neg = polymod_neg;
+    f->is0 = polymod_is0;
+    f->is1 = polymod_is1;
+    f->set0 = polymod_set0;
+    f->set1 = polymod_set1;
+    f->cmp = polymod_cmp;
     f->mul = polymod_mul;
     f->random = polymod_random;
     f->from_hash = polymod_from_hash;
@@ -739,11 +952,10 @@ void field_init_polymod(field_ptr f, element_ptr poly)
     } else {
 	f->fixed_length_in_bytes = pdp->field->fixed_length_in_bytes * poly_degree(poly);
     }
-    n = pd->coeff->count;
-    mpz_pow_ui(f->order, p->field->order, n - 1);
+    mpz_pow_ui(f->order, p->field->order, p->n);
 
-    darray_init(p->xpwr);
-    compute_x_powers(p->xpwr, poly);
+    p->xpwr = malloc(sizeof(element_t) * p->n);
+    compute_x_powers(f, poly);
 }
 
 void trial_divide(darray_ptr factor, darray_ptr mult, mpz_t n)
@@ -804,21 +1016,12 @@ void poly_gcd(element_ptr d, element_ptr f, element_ptr g)
     element_clear(r);
 }
 
-void poly_unmod(element_ptr f)
-    //input: f in K[x]_{p(x)}
-    //output: f is now a polynomial over K[x]
-    //(i.e. the field is not modulo anything)
-{
-    polymod_field_data_ptr p = f->field->data;
-    f->field = p->poly->field;
-}
-
 int poly_is_irred_degfac(element_ptr f, darray_t factor)
     //called by poly_is_irred
     //needs to be passed a list of the factors of deg f
 {
     int res;
-    element_t xpow, x;
+    element_t xpow, x, g;
     mpz_t deg, z;
     int i, n;
     field_ptr basef = poly_base_field(f);
@@ -832,8 +1035,9 @@ int poly_is_irred_degfac(element_ptr f, darray_t factor)
     mpz_set_ui(deg, poly_degree(f));
     element_init(xpow, rxmod);
     element_init(x, rxmod);
+    element_init(g, f->field);
     n = factor->count;
-    poly_setx(x);
+    element_set1(((element_t *) x->data)[1]);
 
     res = 0;
     for (i=0; i<n; i++) {
@@ -844,8 +1048,9 @@ int poly_is_irred_degfac(element_ptr f, darray_t factor)
 	if (element_is0(xpow)) {
 	    goto done;
 	}
-	poly_gcd(xpow, f, xpow);
-	if (poly_degree(xpow) != 0) goto done;
+	element_polymod_to_poly(g, xpow);
+	poly_gcd(g, f, g);
+	if (poly_degree(g) != 0) goto done;
     }
 
     mpz_pow_ui(z, basef->order, poly_degree(f));
@@ -854,6 +1059,7 @@ int poly_is_irred_degfac(element_ptr f, darray_t factor)
     if (element_is0(xpow)) res = 1;
 
 done:
+    element_clear(g);
     element_clear(xpow);
     element_clear(x);
     mpz_clear(deg);
