@@ -7,23 +7,24 @@
 //also R[x]_{f(x)}
 struct poly_field_data_s {
     field_ptr field;
-    fieldmap mapbase;
+    fieldmap mapbase; //map element from underlying field to constant term
 };
 typedef struct poly_field_data_s poly_field_data_t[1];
 typedef struct poly_field_data_s *poly_field_data_ptr;
 
+//TODO: having this structure is unnecessary? what else could be needed besides coeff?
 struct poly_element_s {
     darray_t coeff;
 };
-
 typedef struct poly_element_s poly_element_t[1];
 typedef struct poly_element_s *poly_element_ptr;
 
 struct polymod_field_data_s {
     field_ptr field;
     fieldmap mapbase;
+    int n;
     element_t poly; //polynomial of degree n
-    darray_t xpwr; //holds x^n,...,x^{2n-2} mod poly
+    element_t *xpwr; //holds x^n,...,x^{2n-2} mod poly
 };
 typedef struct polymod_field_data_s polymod_field_data_t[1];
 typedef struct polymod_field_data_s *polymod_field_data_ptr;
@@ -68,10 +69,8 @@ void poly_random_monic(element_ptr f, int deg);
 void poly_make_monic(element_t f, element_t g);
 void element_field_to_poly(element_t poly, element_t constant);
 
-/* this is needed in mnt.c */
-void poly_unmod(element_ptr f);
-    //input: f in K[x]_{p(x)}
-    //output: f is now a polynomial over K[x]
-    //(i.e. the field is not modulo anything)
+void element_polymod_to_poly(element_ptr f, element_ptr e);
+void element_field_to_polymod(element_ptr f, element_ptr a);
+void element_poly_to_polymod_truncate(element_ptr f, element_ptr e);
 
 #endif //POLY_H
